-------
-- Mono Lake Third model: elevation and flow details 
--- 

function createObserverChart (terrame_object, plotLabel)
	chart = Observer {
		subject     = terrame_object,
		attributes  = plotLabel.attributes,
		curveLabels = plotLabel.attributes,
		title       = plotLabel.title,
		xLabel      = plotLabel.xLabel,
		yLabel      = plotLabel.yLabel,
		type = "chart"
	}
end

function createPlot (model)

	model.plot = Cell (model.outcomes)
	createObserverChart (model.plot, model.plotLabels)

end
------------------
function createTimer (model)

	model.timer = Timer {
		Event{time = 0, period = 1, action = function(e)
			changes(model)
			model.plot:notify(e:getTime()) 
		end}
	}
end
-----------
specificGravityL         = {1.00,  1.05, 1.10, 1.15, 1.20, 1.25, 1.30, 1.35, 1.40}   -- dimension-less
evaporationRateFactorL   = {1.000, 0.963, 0.926, 0.880, 0.883, 0.785, 0.737, 0.688, 0.640}   -- dimension-less
-- Table 4.2 - in Ford's book 
function evapRateFactor (specificGravity)

 	a = -1.4208080808073591E+00;
 	b = 6.6409259259240558E+00;
 	c = -5.5515151515135548E+00;
 	d = 1.3265993265988749E+00;

 	local evapRateReductionFactor = a + b*specificGravity+ c*specificGravity^2 + d*specificGravity^3 

	return evapRateReductionFactor
end
-------------

function calcEvaporation (model)

	local massFreshwater = model.stocks.waterInTheLake * model.params.massFreshWaterPerKAF

	local specificGravity = (massFreshwater + model.params.dissolvedSolids) / massFreshwater

	local evaporationRate = evapRateFactor (specificGravity) * model.params.freshWaterEvaporationRate

	evaporation   = model.variables.surfaceArea * evaporationRate

	return evaporation
end

-- Table 4. - in Ford's book
waterObsL     = {0,    1000,  2000, 3000, 4000, 5000, 6000, 7000, 8000}   -- KAF
surfaceObsL   = {0,    24.7,  35.3, 48.6, 54.3, 57.2, 61.6, 66.0, 69.9}   -- KA
elevationObsL = { 6224, 6335, 6369, 6392, 6412, 6430, 6447, 6463, 6477}   -- feet
----------------

function estimateSurfaceArea (water)

	a = 8.1414177963836043E-01
	b = 2.5222522481624796E-02
	c = -3.8848123899271551E-06
	d = 2.2744107985861461E-10

	local surfaceArea = a + b*water + c*water^2 + d*water^3 

	return surfaceArea

end

function estimateElevation (water)

	a = 6.2344545468507940E+03
	b = 9.6353533955755757E-02
	c = -1.6848484772520475E-05
	d = 1.0858585974826696E-09

	local elevation = a + b*water + c*water^2 + d*water^3 

	return elevation
end
-------------------

function changes (model)
    
    model.variables.surfaceArea = estimateSurfaceArea (model.stocks.waterInTheLake)
	model.outcomes.elevation    = estimateElevation (model.stocks.waterInTheLake) 

	model.flows.precipitation    = model.variables.surfaceArea * model.params.precipitationRate

	model.flows.evaporation      = calcEvaporation (model)

	model.stocks.waterInTheLake = model.stocks.waterInTheLake 
		+ model.flows.precipitation
		+ model.flows.runoff    -- water from the mountains
		+ model.flows.otherIn
		- model.flows.evaporation
		- model.flows.otherOut
		- model.policy.waterExport

	if (model.stocks.waterInTheLake <= 0 ) then
		model.stocks.waterInTheLake = 0
	end
end

-----------------

function run (model, time)
	model.timer:execute (time)
end

-----------
function SystemDynamicsModel(model)

	-- create Plot
	createPlot (model)

	-- timer
	createTimer (model)

	return model
end
---------
model =  SystemDynamicsModel {
	plotLabels = {
		  attributes = {"elevation"}, 
	      title      = "MonoLake - Elevation", 
	      xLabel     = "years", 
	      yLabel     = "Elevation (feet)"
	},
	policy        = { waterExport = 100     },
	outcomes      = { elevation = 0         },
	stocks        = { waterInTheLake = 2228 },
	variables     = { surfaceArea      = 0  },  
	flows     	  = { evaporation   = 0, 
					  precipitation = 0, 
					  runoff        = 150,
					  otherIn       = 47.6,
					  otherOut      = 33.6
					 },
	params        = { 
	                  precipitationRate = 0.667, 
	                  freshWaterEvaporationRate = 3.75, 
	                  massFreshWaterPerKAF = 1.359, 
	                  dissolvedSolids = 230
	              }
}

-----
run (model, 40) 


